(*----------------------------------------------------------------------------*
 *  Direct3D sample from DirectX 9.0 SDK Summer 2003                          *
 *  Delphi adaptation by Alexey Barkovoy (e-mail: clootie@ixbt.com)           *
 *                                                                            *
 *  Delphi versions 5-7 are supported                                         *
 *                                                                            *
 *  Modified: 02-Feb-2004                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru/delphi                                         *
 *----------------------------------------------------------------------------*)
//-----------------------------------------------------------------------------
// File: SHIrradianceEnvMap.cpp
//
// Desc: 
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------

unit SHIrradianceEnvMapUnit;

interface

uses
  Windows, Messages,
  Direct3D9, D3DX9, dxerr9,
  D3DApp, D3DFont, D3DUtil, DXUtil,
  Resource;

// C++Builder compile hack
{$NOINCLUDE Resource}
{$HPPEMIT '#include "Resource.hpp"'}

{.$DEFINE DEBUG_VS}   // Uncomment this line to debug vertex shaders
{.$DEFINE DEBUG_PS}   // Uncomment this line to debug pixel shaders


//-----------------------------------------------------------------------------
// Globals variables and definitions
//-----------------------------------------------------------------------------
const
  MAX_LIGHTS = 3;
  g_fSqrtPI = 1.7724538510212332182745076025231; // Sqrt(D3DX_PI);

type
  //-----------------------------------------------------------------------------
  // Name: class CMyD3DApplication
  // Desc: Main class to run this application. Most functionality is inherited
  //       from the CD3DApplication base class.
  //-----------------------------------------------------------------------------
  CMyD3DApplication = class(CD3DApplication)
  private
    m_pFont: ID3DXFont;              // Font for drawing text
    m_bShowHelp: Boolean;
    m_Camera: CModelViewerCamera;
    m_pSphere: ID3DXMesh;

    m_pSimpleLightingEffect: ID3DXEffect;
    m_pIrradianceEnvMapEffect: ID3DXEffect;

    m_LightArcBall:      array[0..MAX_LIGHTS-1] of CD3DArcBall;
    m_vDefaultLightDir:  array[0..MAX_LIGHTS-1] of TD3DXVector3;
    m_vLightDir:         array[0..MAX_LIGHTS-1] of TD3DXVector3;
    m_mLightRot:         array[0..MAX_LIGHTS-1] of TD3DXMatrixA16;
    m_mLightRotSnapshot: array[0..MAX_LIGHTS-1] of TD3DXMatrixA16;
    m_fLightScale: Single;
    m_nNumActiveLights: Integer;
    m_nActiveLight: Integer;

    m_mWorld:          TD3DXMatrixA16;
    m_mView:           TD3DXMatrixA16;
    m_mProj:           TD3DXMatrixA16;
    m_nTechnique:      Integer;

    // m_mtrl:            TD3DMaterial9; //todo: Report as a bug
    m_pTexture:        IDirect3DTexture9;
    m_pMesh:           ID3DXMesh;

  protected
    function OneTimeSceneInit: HRESULT; override;
    function InitDeviceObjects: HRESULT; override;
    function RestoreDeviceObjects: HRESULT; override;
    function InvalidateDeviceObjects: HRESULT; override;
    function DeleteDeviceObjects: HRESULT; override;
    function FinalCleanup: HRESULT; override;
    function Render: HRESULT; override;
    function FrameMove: HRESULT; override;
    function ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
      adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT; override;

    function LoadMesh(strFileName: PChar; out ppMesh: ID3DXMesh): HRESULT;
    function UpdateLightDir: HRESULT;
    function RenderLightSphere(lightDir: TD3DXVector3; sphereColor: TD3DXColor): HRESULT;
    function RenderMeshWithIrradianceEnvMap: HRESULT;
    function RenderMeshWithSimpleLighting: HRESULT;
    function RenderText: HRESULT;

  public
    constructor Create;
    destructor Destroy; override;
    function MsgProc(hWnd: HWND; uMsg: LongWord; wParam: WPARAM; lParam: LPARAM): LRESULT; override;
  end;



implementation

uses SysUtils;

const
  UnitName = 'SHIrradianceEnvMapUnit.pas';

{ CMyD3DApplication }

//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Application constructor.   Paired with ~CMyD3DApplication()
//       Member variables should be initialized to a known state here.  
//       The application window has not yet been created and no Direct3D device 
//       has been created, so any initialization that depends on a window or 
//       Direct3D should be deferred to a later stage. 
//-----------------------------------------------------------------------------
constructor CMyD3DApplication.Create;
var
  i: Integer;
begin
  inherited Create;
  m_Camera:= CModelViewerCamera.Create;
  for i:= 0 to MAX_LIGHTS - 1 do
    m_LightArcBall[i]:= CD3DArcBall.Create;

  m_strWindowTitle := 'SHIrradianceEnvMap';
  m_d3dEnumeration.AppUsesDepthBuffer := True;
  m_pFont := nil;
  m_dwCreationWidth  := 640;
  m_dwCreationHeight := 480;
  m_bShowCursorWhenFullscreen := True;
  m_nTechnique := 0;

  m_vDefaultLightDir[0] := D3DXVector3(0.37,0.36,-0.86);
  m_vDefaultLightDir[1] := D3DXVector3(-0.69,-0.60,-0.42);
  m_vDefaultLightDir[2] := D3DXVector3(-0.85,0.37,0.37);
  for i:= 0 to MAX_LIGHTS - 1 do
  begin
    m_vLightDir[i] := m_vDefaultLightDir[i];
    D3DXMatrixIdentity(m_mLightRot[i]);
    D3DXMatrixIdentity(m_mLightRotSnapshot[i]);
  end;
  m_nActiveLight := 0;
  m_nNumActiveLights := 3;
  m_fLightScale := 1.0;

  m_pIrradianceEnvMapEffect := nil;
  m_pSimpleLightingEffect := nil;

  m_pMesh := nil;
  m_bShowHelp := True;
end;

destructor CMyD3DApplication.Destroy;
var
  i: Integer;
begin
  for i:= 0 to MAX_LIGHTS - 1 do
    FreeAndNil(m_LightArcBall[i]);
  FreeAndNil(m_Camera);
  inherited;
end;


//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Paired with FinalCleanup().
//       The window has been created and the IDirect3D9 interface has been
//       created, but the device has not been created yet.  Here you can
//       perform application-related initialization and cleanup that does
//       not depend on a device.
//-----------------------------------------------------------------------------
function CMyD3DApplication.OneTimeSceneInit: HRESULT;
begin
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Paired with DeleteDeviceObjects()
//       The device has been created.  Resources that are not lost on
//       Reset() can be created here -- resources in D3DPOOL_MANAGED,
//       D3DPOOL_SCRATCH, or D3DPOOL_SYSTEMMEM.  Image surfaces created via
//       CreateImageSurface are never lost and can be created here.  Vertex
//       shaders and pixel shaders can also be created here as they are not
//       lost on Reset().
//-----------------------------------------------------------------------------
function CMyD3DApplication.InitDeviceObjects: HRESULT;
var
  hDC: Windows.HDC;
  nHeight: Integer;
  dwShaderFlags: DWORD;
  str: array[0..MAX_PATH-1] of Char;
begin
  // Initialize the font
  hDC := GetDC(0);
  nHeight := -MulDiv(9, GetDeviceCaps(hDC, LOGPIXELSY), 72);
  ReleaseDC(0, hDC);
  Result := D3DXCreateFont(m_pd3dDevice, nHeight, 0, FW_BOLD, 0, False, DEFAULT_CHARSET,
                           OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH or FF_DONTCARE,
                           'Arial', m_pFont);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('D3DXCreateFont', Result, UnitName);
    Exit;
  end;

  // Load the mesh
  Result := LoadMesh('tiger.x', m_pMesh);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('LoadMesh', Result, UnitName);
    Exit;
  end;

  DXUtil_FindMediaFileCb(str, SizeOf(str), 'tiger.bmp');
  Result := D3DXCreateTextureFromFile(m_pd3dDevice, str, m_pTexture);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('D3DXCreateTextureFromFile', Result, UnitName);
    Exit;
  end;

  // Create a sphere to show the light's position
  Result := D3DXCreateSphere(m_pd3dDevice, 1.0, 8, 8, m_pSphere, nil);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('D3DXCreateSphere', Result, UnitName);
    Exit;
  end;

  // Define DEBUG_VS and/or DEBUG_PS to debug vertex and/or pixel shaders with the shader debugger.
  // Debugging vertex shaders requires either REF or software vertex processing, and debugging
  // pixel shaders requires REF.  The D3DXSHADER_FORCE_*_SOFTWARE_NOOPT flag improves the debug
  // experience in the shader debugger.  It enables source level debugging, prevents instruction
  // reordering, prevents dead code elimination, and forces the compiler to compile against the next
  // higher available software target, which ensures that the unoptimized shaders do not exceed
  // the shader model limitations.  Setting these flags will cause slower rendering since the shaders
  // will be unoptimized and forced into software.  See the DirectX documentation for more information
  // about using the shader debugger.
  dwShaderFlags := 0;
  {$IFDEF DEBUG_VS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_VS_SOFTWARE_NOOPT;
  {$ENDIF}
  {$IFDEF DEBUG_PS}
    dwShaderFlags := dwShaderFlags or D3DXSHADER_FORCE_PS_SOFTWARE_NOOPT;
  {$ENDIF}

  // Read the D3DX effect file
  DXUtil_FindMediaFileCb(str, SizeOf(str), 'SHIrradianceEnvMap.fx');
  Result := D3DXCreateEffectFromFile(m_pd3dDevice, str, nil, nil,
                                     dwShaderFlags, nil, m_pIrradianceEnvMapEffect, nil);

  // If this fails, there should be debug output as to
  // they the .fx file failed to compile
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('D3DXCreateEffectFromFile', Result, UnitName);
    Exit;
  end;

  // Read the D3DX effect file
  DXUtil_FindMediaFileCb(str, SizeOf(str), 'SimpleLighting.fx');
  Result:= D3DXCreateEffectFromFile(m_pd3dDevice, str, nil, nil,
                                 dwShaderFlags, nil, m_pSimpleLightingEffect, nil);

  // If this fails, there should be debug output as to
  // they the .fx file failed to compile
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('D3DXCreateEffectFromFile', Result, UnitName);
    Exit;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: LoadMesh()
// Desc: Load the mesh and ensure the mesh has normals, and optimize
//       the mesh for this graphics card's vertex cache so the triangle list
//       inside in the mesh will cache miss less which improves perf.
//-----------------------------------------------------------------------------
function CMyD3DApplication.LoadMesh(strFileName: PChar; out ppMesh: ID3DXMesh): HRESULT;
var
  pMesh: ID3DXMesh;
  str: array[0..MAX_PATH-1] of Char;
  rgdwAdjacency: PDWORD;
  pTempMesh: ID3DXMesh;
begin
  if (@ppMesh = nil) then
  begin
    Result:= E_INVALIDARG;
    Exit;
  end;

  // Load the mesh with D3DX and get back a ID3DXMesh*.  For this
  // sample we'll ignore the X file's embedded materials since we know
  // exactly the model we're loading.  See the mesh samples such as
  // "OptimizedMesh" for a more generic mesh loading example.
  Result := DXUtil_FindMediaFileCb(str, SizeOf(str), strFileName);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('DXUtil_FindMediaFileCb', D3DAPPERR_MEDIANOTFOUND, UnitName);
    Exit;
  end;
  Result := D3DXLoadMeshFromX(str, D3DXMESH_MANAGED,
                              m_pd3dDevice, nil, nil, nil, nil, pMesh);
  if FAILED(Result) or (pMesh = nil) then
  begin
    Result:= DXTRACE_ERR('D3DXLoadMeshFromX', Result, UnitName);
    Exit;
  end;

  // Make sure there are normals which are required for lighting
  if (pMesh.GetFVF and D3DFVF_NORMAL = 0) then
  begin
    Result := pMesh.CloneMeshFVF(pMesh.GetOptions,
                                 pMesh.GetFVF or D3DFVF_NORMAL,
                                 m_pd3dDevice, pTempMesh);
    if FAILED(Result) then
    begin
      Result:= DXTRACE_ERR('CloneMeshFVF', Result, UnitName);
      Exit;
    end;

    D3DXComputeNormals(pTempMesh, nil);

    SAFE_RELEASE(pMesh);
    pMesh := pTempMesh;
  end;

  // Optimize the mesh for this graphics card's vertex cache
  // so when rendering the mesh's triangle list the vertices will
  // cache hit more often so it won't have to re-execute the vertex shader
  // on those vertices so it will improves perf.
  try
    GetMem(rgdwAdjacency, SizeOf(DWORD) * pMesh.GetNumFaces * 3);
  except
    Result:= E_OUTOFMEMORY;
    Exit;
  end;
  pMesh.ConvertPointRepsToAdjacency(nil, rgdwAdjacency);
  Result:= pMesh.OptimizeInplace(D3DXMESHOPT_VERTEXCACHE, rgdwAdjacency, nil, nil, nil);
  FreeMem(rgdwAdjacency);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('OptimizeInplace', Result, UnitName);
    Exit;
  end;

  ppMesh := pMesh;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Paired with InvalidateDeviceObjects()
//       The device exists, but may have just been Reset().  Resources in
//       D3DPOOL_DEFAULT and any other device state that persists during
//       rendering should be set here.  Render states, matrices, textures,
//       etc., that don't change during rendering can be set once here to
//       avoid redundant state setting during Render() or FrameMove().
//-----------------------------------------------------------------------------
function CMyD3DApplication.RestoreDeviceObjects: HRESULT;
var
  i: Integer;
  fAspect: Single;
  vFrom: TD3DXVector3;
  vAt: TD3DXVector3;
begin
  if Assigned(m_pFont) then
  begin
    Result:= m_pFont.OnResetDevice;
    if FAILED(Result) then
    begin
      Result:= DXTRACE_ERR('m_pFont.OnResetDevice', Result, UnitName);
      Exit;
    end;
  end;

  for i:= 0 to MAX_LIGHTS - 1 do
    m_LightArcBall[i].SetWindow(m_d3dsdBackBuffer.Width, m_d3dsdBackBuffer.Height);

  // Turn off lighting since its done in the shaders
  m_pd3dDevice.SetRenderState(D3DRS_LIGHTING, iFalse);

  // Setup the camera
  fAspect := m_d3dsdBackBuffer.Width / m_d3dsdBackBuffer.Height;
  m_Camera.SetProjParams(D3DX_PI/4, fAspect, 1.0, 10000.0);
  vFrom:= D3DXVector3(0.0, 0.0, -5.0);
  vAt:= D3DXVector3Zero;
  m_Camera.SetViewParams(vFrom, vAt);
  m_Camera.SetRadius(5.0, 1.0, 20.0);
  m_Camera.SetButtonMasks(MOUSE_LEFT_BUTTON, MOUSE_WHEEL, MOUSE_MIDDLE_BUTTON);
  m_Camera.SetWindow(m_d3dsdBackBuffer.Width, m_d3dsdBackBuffer.Height);
  m_Camera.SetEnablePositionMovement(False);
  m_Camera.SetResetCursorAfterMove(False);

  if Assigned(m_pSimpleLightingEffect) then m_pSimpleLightingEffect.OnResetDevice;
  if Assigned(m_pIrradianceEnvMapEffect) then m_pIrradianceEnvMapEffect.OnResetDevice;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FrameMove: HRESULT;
begin
  // Update the camera's postion based on user input
  m_Camera.FrameMove(m_fElapsedTime);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.Render: HRESULT;
var
  mWorldViewProj, m: TD3DXMatrixA16;
  i: Integer;
  sphereColor: TD3DXColor;
begin
  // Get the projection & view matrix from the camera class
  m_mView  := m_Camera.GetViewMatrix^;
  m_mProj  := m_Camera.GetProjMatrix^;
  m_mWorld := m_Camera.GetWorldMatrix^;

  // Clear the render target
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER, $00003F3F, 1.0, 0);

  // Begin the scene
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    // Render the light spheres so the user can
    // visually see the light dir
    for i:= 0 to m_nNumActiveLights - 1 do
    begin
      if (i = m_nActiveLight)
        then sphereColor := D3DXColor(1,1,0,1)
        else sphereColor := D3DXColor(1,1,1,1);
      RenderLightSphere(m_vLightDir[i], sphereColor);
    end;

    // mWorldViewProj = m_mWorld * m_mView * m_mProj;
    D3DXMatrixMultiply(m, m_mView, m_mProj);
    D3DXMatrixMultiply(mWorldViewProj, m_mWorld, m);

    m_pIrradianceEnvMapEffect.SetMatrix('mWorldViewProjection', mWorldViewProj);
    m_pSimpleLightingEffect.SetMatrix('mWorldViewProjection', mWorldViewProj);

    case m_nTechnique of
      0:
      begin
        Result := RenderMeshWithIrradianceEnvMap;
        if FAILED(Result) then Exit;
      end;
      1:
      begin
        Result := RenderMeshWithSimpleLighting;
        if FAILED(Result) then Exit;
      end;
    end;

    RenderText;

    // End the scene.
    Result := m_pd3dDevice.EndScene;
    if FAILED(Result) then
    begin
      Result:= DXTRACE_ERR('m_pd3dDevice.EndScene', Result, UnitName);
      Exit;
    end;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RenderLightSphere()
// Desc: Render the light spheres so the user can visually see the light dir
//-----------------------------------------------------------------------------
function CMyD3DApplication.RenderLightSphere(lightDir: TD3DXVector3; sphereColor: TD3DXColor): HRESULT;
var
  iPass, cPasses: LongWord;
  mScale: TD3DXMatrix;
  mTrans: TD3DXMatrix;
  mWorldViewProj: TD3DXMatrixA16;
  lightOff: TD3DXColor;
  fObjectRadius: Single;
  vL: TD3DXVector3;
  m: TD3DXMatrix;
  mtrlAmbientColor: TD3DXColor;
begin
  m_pSimpleLightingEffect.SetTechnique('SimpleLightingTex0');

  lightOff:= D3DXColor(0,0,0,0);
  m_pSimpleLightingEffect.SetVector('Light1DiffuseColor',   PD3DXVector4(@lightOff)^);
  m_pSimpleLightingEffect.SetVector('Light2DiffuseColor',   PD3DXVector4(@lightOff)^);
  m_pSimpleLightingEffect.SetVector('Light3DiffuseColor',   PD3DXVector4(@lightOff)^);
  m_pSimpleLightingEffect.SetVector('MaterialAmbientColor', PD3DXVector4(@sphereColor)^);

  fObjectRadius := 1.5;
  D3DXVec3Scale(vL, lightDir, fObjectRadius * 1.2);
  D3DXMatrixTranslation(mTrans, vL.x, vL.y, vL.z);
  D3DXMatrixScaling(mScale, fObjectRadius*0.05, fObjectRadius*0.05, fObjectRadius*0.05);

  // mWorldViewProj = mScale * mTrans * m_mView * m_mProj;
  D3DXMatrixMultiply(m, m_mView, m_mProj);
  D3DXMatrixMultiply(m, mTrans, m);
  D3DXMatrixMultiply(mWorldViewProj, mScale, m);

  m_pSimpleLightingEffect.SetMatrix('mWorldViewProjection', mWorldViewProj);

  Result := m_pSimpleLightingEffect._Begin(@cPasses, 0);
  if FAILED(Result) then
  begin
    Result:= DXTRACE_ERR('m_pSimpleLightingEffect._Begin', Result, UnitName);
    Exit;
  end;
  for iPass := 0 to cPasses - 1 do
  begin
    Result := m_pSimpleLightingEffect.Pass(iPass);
    if FAILED(Result) then
    begin
      Result:= DXTRACE_ERR('m_pSimpleLightingEffect.Pass', Result, UnitName);
      Exit;
    end;
    Result := m_pSphere.DrawSubset(0);
    if FAILED(Result) then
    begin
      Result:= DXTRACE_ERR('DrawSubset', Result, UnitName);
      Exit;
    end;
  end;
  m_pSimpleLightingEffect._End;

  mtrlAmbientColor := D3DXColor(0,0,0,0);
  m_pSimpleLightingEffect.SetVector('MaterialAmbientColor', PD3DXVector4(@mtrlAmbientColor)^);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RenderMeshWithIrradianceEnvMap()
// Desc:
//-----------------------------------------------------------------------------
function CMyD3DApplication.RenderMeshWithIrradianceEnvMap: HRESULT;
var
  iPass, cPasses, iChannel: LongWord;
  mWorldInv: TD3DXMatrix;
const
  // This sample uses order 3 so they are 3^2 coefficents per channel and 3 channels (R,G,B)
  // If the light had the same intensity for red, green, and blue we could optimize
  // and just use a single channel (the red channel)
  dwOrder = 3;
var
  fLight: array[0..MAX_LIGHTS-1, 0..2, 0..dwOrder*dwOrder-1] of Single;
  lightColor: TD3DXColor;
  lightDirObjectSpace: TD3DXVector3;
  i: Integer;
  // Lighting environment coefficients
  vCoefficients: array[0..2] of TD3DXVector4;
  fC0, fC1, fC2, fC3, fC4: Single;
begin
  D3DXMatrixInverse(mWorldInv, nil, m_mWorld);

  lightColor := D3DXColor(1.0,1.0,1.0,1.0);

  D3DXColorScale(lightColor, lightColor, m_fLightScale);

  for i:= 0 to m_nNumActiveLights - 1 do
  begin
    // Transform the world space light dir into object space
    // Note that if there's multiple objects using PRT in the scene, then
    // for each object you need to either evaulate the lights in object space
    // evaulate the lights in world and rotate the light coefficients
    // into object space.
    D3DXVec3TransformNormal(lightDirObjectSpace, m_vLightDir[i], mWorldInv);

    // Pass in the light direction, the intensity of each channel, and it returns
    // an array of source radiance coefficients for each channel.
    // For order 3, there are 9 coefficients per channel.
    Result := D3DXSHEvalDirectionalLight(3, lightDirObjectSpace,
                                    lightColor.r, lightColor.g, lightColor.b,
                                    @fLight[i][0], @fLight[i][1], @fLight[i][2]);
    if FAILED(Result) then
    begin
      Result:= DXTRACE_ERR('D3DXSHEvalDirectionalLight', Result, UnitName);
      Exit;
    end;
  end;

  // For multiple lights, just them sum up using D3DXSHAdd().
  for i:= 1 to m_nNumActiveLights - 1 do
  begin
    // D3DXSHAdd will add Order^2 floats.  There are 3 color channels,
    // so call it 3 times.
    D3DXSHAdd(@fLight[0][0], dwOrder, @fLight[0][0], @fLight[i][0]);
    D3DXSHAdd(@fLight[0][1], dwOrder, @fLight[0][1], @fLight[i][1]);
    D3DXSHAdd(@fLight[0][2], dwOrder, @fLight[0][2], @fLight[i][2]);
  end;

  // These constants are described in the article by Peter-Pike Sloan titled
  // "Efficient Evaluation of Irradiance Environment Maps" in the book
  // "ShaderX 2 - Shader Programming Tips and Tricks" by Wolfgang F. Engel.
  fC0 := 1.0/(2.0*g_fSqrtPI);
  fC1 := Sqrt(3.0)/(3.0*g_fSqrtPI);
  fC2 := Sqrt(15.0)/(8.0*g_fSqrtPI);
  fC3 := Sqrt(5.0)/(16.0*g_fSqrtPI);
  fC4 := 0.5*fC2;

  for iChannel:= 0 to 2 do
  begin
    vCoefficients[iChannel].x := -fC1*fLight[0][iChannel][3];
    vCoefficients[iChannel].y := -fC1*fLight[0][iChannel][1];
    vCoefficients[iChannel].z :=  fC1*fLight[0][iChannel][2];
    vCoefficients[iChannel].w :=  fC0*fLight[0][iChannel][0] - fC3*fLight[0][iChannel][6];
  end;

  m_pIrradianceEnvMapEffect.SetVector('cAr', vCoefficients[0]);
  m_pIrradianceEnvMapEffect.SetVector('cAg', vCoefficients[1]);
  m_pIrradianceEnvMapEffect.SetVector('cAb', vCoefficients[2]);

  for iChannel:= 0 to 2 do
  begin
    vCoefficients[iChannel].x :=     fC2*fLight[0][iChannel][4];
    vCoefficients[iChannel].y :=    -fC2*fLight[0][iChannel][5];
    vCoefficients[iChannel].z := 3.0*fC3*fLight[0][iChannel][6];
    vCoefficients[iChannel].w :=    -fC2*fLight[0][iChannel][7];
  end;

  m_pIrradianceEnvMapEffect.SetVector('cBr', vCoefficients[0]);
  m_pIrradianceEnvMapEffect.SetVector('cBg', vCoefficients[1]);
  m_pIrradianceEnvMapEffect.SetVector('cBb', vCoefficients[2]);

  vCoefficients[0].x := fC4*fLight[0][0][8];
  vCoefficients[0].y := fC4*fLight[0][1][8];
  vCoefficients[0].z := fC4*fLight[0][2][8];
  vCoefficients[0].w := 1.0;

  m_pIrradianceEnvMapEffect.SetVector('cC', vCoefficients[0]);

  m_pIrradianceEnvMapEffect.SetTechnique('IrradianceEnvironmentMap');
  m_pIrradianceEnvMapEffect.SetTexture('MeshTexture', m_pTexture);

  Result := m_pIrradianceEnvMapEffect._Begin(@cPasses, 0);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pEffect.Begin', Result, UnitName); Exit; end;

  for iPass := 0 to cPasses - 1 do
  begin
    Result:= m_pIrradianceEnvMapEffect.Pass(iPass);
    if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pEffect.Pass', Result, UnitName); Exit; end;

    Result:= m_pMesh.DrawSubset(0);
    if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pMesh.DrawSubset', Result, UnitName); Exit; end;
  end;

  m_pIrradianceEnvMapEffect._End;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: RenderMeshWithSimpleLighting()
// Desc: Render the scene with standard diffuse (N.L) lighting
//-----------------------------------------------------------------------------
function CMyD3DApplication.RenderMeshWithSimpleLighting: HRESULT;
var
  iPass, cPasses: LongWord;
  lightDirObjectSpace: TD3DXVector3;
  lightOn: TD3DXColor;
  lightOff: TD3DXColor;
  mWorldInv: TD3DXMatrix;
  dwAttribs: DWORD;
  i: DWORD;
begin
  lightOn:= D3DXColor(1,1,1,1);
  lightOff:= D3DXColor(0,0,0,0);

  D3DXColorScale(lightOn, lightOn, m_fLightScale);

  if (m_pMesh = nil) then
  begin
    Result:= S_OK;
    Exit;
  end;

  m_pSimpleLightingEffect.SetTechnique('SimpleLightingTex1');
  m_pSimpleLightingEffect.SetTexture('MeshTexture', m_pTexture);

  D3DXMatrixInverse(mWorldInv, nil, m_mWorld);

  // Light 1
  D3DXVec3TransformNormal(lightDirObjectSpace, m_vLightDir[0], mWorldInv);
  m_pSimpleLightingEffect.SetFloatArray('Light1Dir', @lightDirObjectSpace.x, 3);
  m_pSimpleLightingEffect.SetVector('Light1DiffuseColor', PD3DXVector4(@lightOn)^);

  // Light 2
  D3DXVec3TransformNormal(lightDirObjectSpace, m_vLightDir[1], mWorldInv);
  m_pSimpleLightingEffect.SetFloatArray('Light2Dir', @lightDirObjectSpace.x, 3);
  if (m_nNumActiveLights > 1)
    then m_pSimpleLightingEffect.SetVector('Light2DiffuseColor', PD3DXVector4(@lightOn)^)
    else m_pSimpleLightingEffect.SetVector('Light2DiffuseColor', PD3DXVector4(@lightOff)^);

  // Light 3
  D3DXVec3TransformNormal(lightDirObjectSpace, m_vLightDir[2], mWorldInv);
  m_pSimpleLightingEffect.SetFloatArray('Light3Dir', @lightDirObjectSpace.x, 3);
  if (m_nNumActiveLights > 2)
    then m_pSimpleLightingEffect.SetVector('Light3DiffuseColor', PD3DXVector4(@lightOn)^)
    else m_pSimpleLightingEffect.SetVector('Light3DiffuseColor', PD3DXVector4(@lightOff)^);

  // Render the scene using simple N dot L lighting (up to 3 lights)
  Result := m_pSimpleLightingEffect._Begin(@cPasses, 0);
  if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pSimpleLightingEffect.Begin', Result, UnitName); Exit; end;

  for iPass := 0 to cPasses - 1 do
  begin
    Result := m_pSimpleLightingEffect.Pass(iPass);
    if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pSimpleLightingEffect.Pass', Result, UnitName); Exit; end;

    dwAttribs := 0;
    m_pMesh.GetAttributeTable(nil, @dwAttribs);
    for i:= 0 to dwAttribs - 1 do
    begin
      Result := m_pMesh.DrawSubset(i);
      if FAILED(Result) then begin Result:= DXTRACE_ERR('m_pMesh.DrawSubset', Result, UnitName); Exit; end;
    end;
  end;
  m_pSimpleLightingEffect._End;

  Result:= S_OK;
end;




//-----------------------------------------------------------------------------
// Name: RenderText()
// Desc: Draw the help & statistics for running sample
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Name: RenderText()
// Desc: Draw the help & statistics for running sample
//-----------------------------------------------------------------------------
function CMyD3DApplication.RenderText: HRESULT;
var
  rc: TRect;
  iLineY: Integer;
  sz: array[0..99] of Char;
begin
  if (nil = m_pFont) then
  begin
    Result:= S_FALSE;
    Exit;
  end;

{$IFDEF UNICODE}
  For UNICODE you should use "m_pFont.DrawTextW" routine!!!
{$ENDIF}

  // Output statistics
  iLineY := 0;
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result:= m_pFont.DrawTextA(nil, m_strFrameStats, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 0.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result:= m_pFont.DrawTextA(nil, m_strDeviceStats, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 0.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

  case m_nTechnique of
    0: StrCopy(sz, 'Technique: RenderMeshWithIrradianceEnvMap');
    1: StrCopy(sz, 'Technique: RenderMeshWithSimpleLighting');
  end;
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result:= m_pFont.DrawTextA(nil, sz, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

  StrLFmt(sz, 256, 'Number of lights: %d, Light scale: %0.2f, Light dir: <%0.2f,%0.2f,%0.2f>',
    [m_nNumActiveLights, m_fLightScale, m_vLightDir[m_nActiveLight].x, m_vLightDir[m_nActiveLight].y, m_vLightDir[m_nActiveLight].z]);
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result:= m_pFont.DrawTextA(nil, sz, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

  case m_nTechnique of
    0: StrCopy(sz, 'This is SHIrrEnvMap lighting.  Press ''N'' to change to simple N.L');
    1: StrCopy(sz, 'This is simple N.L lighting.  Press ''N'' to change to SHIrrEnvMap');
  end;
  SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
  Result:= m_pFont.DrawTextA(nil, sz, -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

  // Draw help
  if m_bShowHelp then
  begin
    iLineY := m_d3dsdBackBuffer.Height-15*6;
    SetRect(rc, 2, iLineY, 0, 0); Inc(iLineY, 15);
    Result:= m_pFont.DrawTextA(nil, 'Controls:', -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));
      if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;
    SetRect(rc, 20, iLineY, 0, 0);
    Result:= m_pFont.DrawTextA(nil, 'Rotate model: Left mouse button'#10 +
                                    'Rotate camera: Right mouse button'#10 +
                                    'Zoom camera: Mouse wheel scroll'#10 +
                                    'Hide help: F1'#10,
                               -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

    SetRect(rc, 250, iLineY, 0, 0);
    Result:= m_pFont.DrawTextA(nil, 'Change # of active lights: L'#10 +
                                    'Change active light: K'#10 +
                                    'Change light scale: U,I,O'#10,
                               -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;

    SetRect(rc, 450, iLineY, 0, 0);
    Result:= m_pFont.DrawTextA(nil, 'Hide help: F1'#10 +
                                    'Change Device: F2'#10 +
                                    'View readme: F9'#10 +
                                    'Quit: ESC',
                               -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 0.75, 0.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;
  end else
  begin
    SetRect(rc, 2, iLineY, 0, 0); // Inc(iLineY, 15); //BAA: never used anyway
    Result:= m_pFont.DrawTextA(nil, 'Press F1 for help',
                               -1, @rc, DT_NOCLIP, D3DXColorToDWord(D3DXColor(1.0, 1.0, 1.0, 1.0)));
    if FAILED(Result) then begin Result:= DXTRACE_ERR('DrawText', Result, UnitName); Exit; end;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Invalidates device objects.  Paired with RestoreDeviceObjects()
//-----------------------------------------------------------------------------
function CMyD3DApplication.InvalidateDeviceObjects: HRESULT;
begin
  if Assigned(m_pFont) then m_pFont.OnLostDevice;
  if Assigned(m_pSimpleLightingEffect) then m_pSimpleLightingEffect.OnLostDevice;
  if Assigned(m_pIrradianceEnvMapEffect) then m_pIrradianceEnvMapEffect.OnLostDevice;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Paired with InitDeviceObjects()
//       Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.DeleteDeviceObjects: HRESULT;
begin
  SAFE_RELEASE(m_pFont);
  SAFE_RELEASE(m_pMesh);
  SAFE_RELEASE(m_pSphere);
  SAFE_RELEASE(m_pTexture);
  SAFE_RELEASE(m_pSimpleLightingEffect);
  SAFE_RELEASE(m_pIrradianceEnvMapEffect);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Paired with OneTimeSceneInit()
//       Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FinalCleanup: HRESULT;
begin
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
function CMyD3DApplication.ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
  adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT;
begin
  Result:= E_FAIL;

  // Debugging vertex shaders requires either REF or software vertex processing
  // and debugging pixel shaders requires REF.
  {$IFDEF DEBUG_VS}
  if (pCaps.DeviceType <> D3DDEVTYPE_REF) and
     ((dwBehavior and D3DCREATE_SOFTWARE_VERTEXPROCESSING) = 0) then Exit;
  {$ENDIF}
  {$IFDEF DEBUG_PS}
  if (pCaps.DeviceType <> D3DDEVTYPE_REF) then Exit;
  {$ENDIF}

  // If device doesn't support 1.1 vertex shaders in HW, switch to SWVP.
  if (pCaps.VertexShaderVersion < D3DVS_VERSION(1,1)) then
  begin
    if ((dwBehavior and D3DCREATE_SOFTWARE_VERTEXPROCESSING) = 0) then Exit;
  end;

  // No fallback, so need ps1.1
  if (pCaps.PixelShaderVersion < D3DPS_VERSION(1,1)) then Exit;

  Result:= S_OK;
end;



//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: Overrrides the main WndProc, so the sample can do custom message
//       handling (e.g. processing mouse, keyboard, or menu commands).
//-----------------------------------------------------------------------------
var
  s_fRemember: Single = 0.0;

function CMyD3DApplication.MsgProc(hWnd: HWND; uMsg: LongWord;
  wParam: WPARAM; lParam: LPARAM): LRESULT;
var
  iMouseX: Integer;
  iMouseY: Integer;
begin
  m_Camera.HandleMessages(hWnd, uMsg, wParam, lParam);

  case uMsg of
    WM_COMMAND:
      case LOWORD(wParam) of
        IDM_TOGGLEHELP: m_bShowHelp := not m_bShowHelp;
      end;

    WM_KEYDOWN:
      case Char(wParam) of
        'N':
        begin
          Inc(m_nTechnique);
          m_nTechnique := m_nTechnique mod 2;
        end;

        'K':
        if not m_LightArcBall[m_nActiveLight].IsBeingDragged then
        begin
          Inc(m_nActiveLight);
          m_nActiveLight := m_nActiveLight mod m_nNumActiveLights;
        end;

        'L':
        if not m_LightArcBall[m_nActiveLight].IsBeingDragged then
        begin
          m_nNumActiveLights := m_nNumActiveLights mod MAX_LIGHTS;
          Inc(m_nNumActiveLights);
          m_nActiveLight := m_nActiveLight mod m_nNumActiveLights;
          UpdateLightDir;
        end;

        'I': m_fLightScale := m_fLightScale * 1.25;
        'U': m_fLightScale := m_fLightScale * 0.8; 
        'O': m_fLightScale := 1.0;
      end;

    WM_RBUTTONDOWN:
    begin
      iMouseX := LOWORD(lParam); // GET_X_LPARAM(lParam);
      iMouseY := HIWORD(lParam); // GET_Y_LPARAM(lParam);
      m_LightArcBall[m_nActiveLight].OnBegin(iMouseX, iMouseY);
      SetCapture(hWnd);
      Result:= iTrue;
      Exit;
    end;

    WM_MOUSEMOVE:
    begin
      iMouseX := LOWORD(lParam); // GET_X_LPARAM(lParam);
      iMouseY := HIWORD(lParam); // GET_Y_LPARAM(lParam);
      if m_LightArcBall[m_nActiveLight].IsBeingDragged
      then m_LightArcBall[m_nActiveLight].OnMove(iMouseX, iMouseY);

      if m_Camera.IsBeingDragged then
      begin
        // Evaluate the lights in terms of SH coefficients
        // and set the shader constants, and call this function
        // whenever the lights or object rotates and need to be reevaluated
        UpdateLightDir;
      end;
      Result:= iTrue;
      Exit;
    end;

    WM_LBUTTONUP,
    WM_MBUTTONUP,
    WM_RBUTTONUP:
    begin
      if (uMsg = WM_RBUTTONUP) then
      begin
        m_LightArcBall[m_nActiveLight].OnEnd;
        ReleaseCapture;
      end;

      UpdateLightDir;
      Result:= iTrue;
      Exit;
    end;
  end;

  Result:= inherited MsgProc(hWnd, uMsg, wParam, lParam);
end;


//-----------------------------------------------------------------------------
// Name: UpdateLightDir()
// Desc: Make the arcball respond correctly when the camera has be rotated.
//-----------------------------------------------------------------------------
function CMyD3DApplication.UpdateLightDir: HRESULT;
var
  m, mInvView, mLastRotInv, mRot: TD3DXMatrix;
  pXBasis, pYBasis, pZBasis: PD3DXVector3;
begin
  D3DXMatrixInverse(mInvView, nil, m_mView);
  mInvView._41 := 0; mInvView._42 := 0; mInvView._43 := 0;

  D3DXMatrixInverse(mLastRotInv, nil, m_mLightRotSnapshot[m_nActiveLight]);

  mRot := m_LightArcBall[m_nActiveLight].GetRotationMatrix^;
  m_mLightRotSnapshot[m_nActiveLight] := mRot;

  // Accumulate the delta of the arcball's rotation in view space.
  // Note that per-frame delta rotations could be problematic over long periods of time.
  // m_mLightRot[m_nActiveLight] *= m_mView * mLastRotInv * mRot * mInvView;
  D3DXMatrixMultiply(m, mRot, mInvView);
  D3DXMatrixMultiply(m, mLastRotInv, m);
  D3DXMatrixMultiply(m, m_mView, m);
  D3DXMatrixMultiply(m_mLightRot[m_nActiveLight], m_mLightRot[m_nActiveLight], m);

  // Since we're accumulating delta rotations, we need to orthonormalize
  // the matrix to prevent eventual matrix skew
  pXBasis := PD3DXVector3(@m_mLightRot[m_nActiveLight]._11);
  pYBasis := PD3DXVector3(@m_mLightRot[m_nActiveLight]._21);
  pZBasis := PD3DXVector3(@m_mLightRot[m_nActiveLight]._31);
  D3DXVec3Normalize(pXBasis^, pXBasis^);
  D3DXVec3Cross(pYBasis^, pZBasis^, pXBasis^);
  D3DXVec3Normalize(pYBasis^, pYBasis^);
  D3DXVec3Cross(pZBasis^, pXBasis^, pYBasis^);

  // Transform the default direction vector by the light's rotation matrix
  D3DXVec3TransformNormal(m_vLightDir[m_nActiveLight], m_vDefaultLightDir[m_nActiveLight], m_mLightRot[m_nActiveLight]);

  Result:= S_OK;
end;

end.

